/*********************************  comphfns.c  *****************************

  Purpose:	Computation of the three h functions associated with a key.

  Provenance:	Written and tested by Q. Chen and E. Fox, March 1991.
  		Edited and tested by S. Wartik, April 1991.

  Notes:	None.

**/


#include <stdio.h>
#include <string.h>

#include "types.h"
#include "rantab.h"


/*************************************************************************

	compute_h012( int, int, randomTablesType, char*, arcType* )

  Return:	void
  
  Purpose:	Compute the triple for a key.  On return, the h0 and h12
  		fields of "arc" have the triple's values.
**/

void compute_h012(n, r, tables, key, arc)
    int		     n,           /* in: number of arcs.                     */
		     r;           /* in: size of h1 or h2 side of the graph. */
    randomTablesType tables;      /* in: pointer to the random tables.       */
    char	     *key;        /* in: key string.                         */
    arcType	     *arc;        /* out: the key's arc entry.	     	     */
{
    int i,          	/* Iterator over each table.			*/
	j,		/* Iterator over each character in "key".	*/
	sum[NO_TABLES],	/* Running sum of h0, h1 and h2 values.		*/
	length;		/* The length of "key".				*/

    length = strlen(key) ;
    sum[0] = sum[1] = sum[2] = 0 ;

    for ( i = 0; i < NO_TABLES; i++ )         /* Sum over all the characters */
	for ( j = 0; j < length; j++ )        /* in the key.                 */
	    sum[i] += tables[i][(key[j]%ROWS)][j];

    arc->h0     = abs( sum[0] ) % n;	      /* Assign mappings for each    */
    arc->h12[0] = abs( sum[1] ) % r;	      /* of h0, h1, and h2 according */
    arc->h12[1] = abs( sum[2] ) % r + r;      /* to the sums computed.	     */
}
